import React, { cloneElement, isValidElement, useEffect, useMemo, useRef, useState } from "react";
import { ExtendedSpeechSynthesis, JSXToArray, JSXToText, findCharIndex, getIndex, isParent, sanitize } from "./utils.js";
export function useSpeech({ text, pitch = 1, rate = 1, volume = 1, lang, voiceURI, preserveUtteranceQueue = false, highlightText = false, highlightProps = { style: { backgroundColor: "yellow" } }, onError = () => alert("Browser not supported! Try some other browser."), onStart, onResume, onPause, onStop, }) {
    const [speechStatus, setSpeechStatus] = useState("stopped");
    const [speakingWord, setSpeakingWord] = useState();
    const utteranceRef = useRef();
    const characters = useMemo(() => JSXToArray(text), [text]);
    const cancel = () => { var _a; return (_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.cancel(); };
    function start() {
        const synth = window.speechSynthesis;
        if (!synth)
            return onError();
        if (speechStatus === "paused")
            return synth.resume();
        if (speechStatus === "queued")
            return;
        const utterance = (utteranceRef.current = new SpeechSynthesisUtterance(sanitize(JSXToText(text))));
        utterance.pitch = pitch;
        utterance.rate = rate;
        utterance.volume = volume;
        if (lang)
            utterance.lang = lang;
        if (voiceURI) {
            if (!Array.isArray(voiceURI))
                voiceURI = [voiceURI];
            const voices = synth.getVoices();
            for (let i = 0; i < voiceURI.length; i++) {
                const uri = voiceURI[i];
                const voice = voices.find((voice) => voice.voiceURI === uri);
                if (voice) {
                    utterance.voice = voice;
                    break;
                }
            }
        }
        const stopEventHandler = (event) => {
            setSpeechStatus("stopped");
            setSpeakingWord(undefined);
            utterance.onstart = null;
            utterance.onresume = null;
            utterance.onpause = null;
            utterance.onend = null;
            utterance.onerror = null;
            utterance.onboundary = null;
            if (synth.paused)
                cancel();
            ExtendedSpeechSynthesis.removeFromQueue();
            ExtendedSpeechSynthesis.speakFromQueue();
            onStop === null || onStop === void 0 ? void 0 : onStop(event);
        };
        utterance.onstart = (event) => {
            setSpeechStatus("started");
            onStart === null || onStart === void 0 ? void 0 : onStart(event);
        };
        utterance.onresume = (event) => {
            setSpeechStatus("started");
            onResume === null || onResume === void 0 ? void 0 : onResume(event);
        };
        utterance.onpause = (event) => {
            setSpeechStatus("paused");
            onPause === null || onPause === void 0 ? void 0 : onPause(event);
        };
        utterance.onend = stopEventHandler;
        utterance.onerror = stopEventHandler;
        if (highlightText)
            utterance.onboundary = ({ charIndex, charLength }) => setSpeakingWord({ index: findCharIndex(characters, charIndex), length: charLength });
        if (!preserveUtteranceQueue)
            ExtendedSpeechSynthesis.clearQueue();
        ExtendedSpeechSynthesis.addToQueue(utterance);
        if (synth.speaking) {
            if (preserveUtteranceQueue) {
                if (speechStatus !== "started")
                    return setSpeechStatus("queued");
                ExtendedSpeechSynthesis.removeFromQueue();
            }
            cancel();
        }
        ExtendedSpeechSynthesis.speakFromQueue();
    }
    function pause() {
        var _a;
        if (speechStatus === "started")
            return (_a = window.speechSynthesis) === null || _a === void 0 ? void 0 : _a.pause();
        if (speechStatus === "queued")
            stop();
    }
    function stop() {
        if (speechStatus === "stopped")
            return;
        if (speechStatus !== "queued")
            return cancel();
        setSpeechStatus("stopped");
        ExtendedSpeechSynthesis.removeFromQueue(utteranceRef.current);
    }
    function highlightedText(element, parentIndex = "") {
        var _a;
        if (!isParent(speakingWord === null || speakingWord === void 0 ? void 0 : speakingWord.index, parentIndex))
            return element;
        if (Array.isArray(element))
            return element.map((child, index) => highlightedText(child, getIndex(parentIndex, index)));
        if (isValidElement(element))
            return cloneElement(element, { key: (_a = element.key) !== null && _a !== void 0 ? _a : Math.random() }, highlightedText(element.props.children, parentIndex));
        if (typeof element === "string" || typeof element === "number") {
            element = element.toString();
            const { index, length } = speakingWord;
            const before = element.slice(0, +index.split("-").at(-1)).length;
            return (React.createElement("span", { key: index },
                element.slice(0, before),
                React.createElement("span", Object.assign({}, highlightProps), element.slice(before, before + length)),
                element.slice(before + length)));
        }
        return element;
    }
    useEffect(() => {
        cancel();
        return cancel;
    }, []);
    return {
        Text: () => highlightedText(text),
        speechStatus,
        start,
        pause,
        stop,
        isInQueue: speechStatus === "started" || speechStatus === "queued",
    };
}
